import pytest

from flask import Flask, current_app
from flask_storm import FlaskStorm, find_flask_storm
from mock import patch
from storm.locals import Store

require = pytest.mark.usefixtures


def test_find_flask_storm(app, flask_storm):
    assert find_flask_storm(app) is flask_storm


def test_duplicate_init(app, flask_storm):
    with pytest.raises(ValueError):
        flask_storm.init_app(app)

    with pytest.raises(ValueError):
        fs = FlaskStorm()
        fs.init_app(app)


def test_already_bound_init_app(app):
    flask_storm = FlaskStorm(app)
    with pytest.raises(RuntimeError):
        flask_storm.init_app(Flask("bar"))


def test_is_bound_init_app(app):
    fs = FlaskStorm()
    assert not fs.is_bound

    fs.init_app(app)
    assert not fs.is_bound


def test_is_bound(app):
    fs = FlaskStorm(app)
    assert fs.is_bound


@require("app_context")
def test_get_binds(app, flask_storm):
    memory_uri = app.config["STORM_DATABASE_URI"]

    assert flask_storm.get_binds() == {None: memory_uri}

    bind_uri = memory_uri + "?extra_param"
    app.config["STORM_BINDS"] = {"extra": bind_uri}
    assert flask_storm.get_binds() == {
        None: memory_uri,
        "extra": bind_uri
    }

@require("app_context")
def test_none_in_binds(app, flask_storm):
    app.config["STORM_BINDS"] = {
        None: app.config["STORM_DATABASE_URI"],
    }

    with pytest.raises(RuntimeError):
        flask_storm.get_binds()


@require("app_context")
def test_connect(app, flask_storm):
    store = flask_storm.connect()
    try:
        assert isinstance(store, Store)
    finally:
        store.close()


@require("app_context")
def test_connect_no_binds(app, flask_storm):
    app.config.pop("STORM_DATABASE_URI")

    with pytest.raises(RuntimeError):
        flask_storm.connect()


@require("app_context")
def test_connect_binds(app, flask_storm):
    # Remove configuration key for test
    memory_uri = app.config.pop("STORM_DATABASE_URI")

    app.config["STORM_BINDS"] = {
        "memory": memory_uri
    }

    with pytest.raises(RuntimeError):
        flask_storm.connect()

    store = flask_storm.connect("memory")
    try:
        assert isinstance(store, Store)
    finally:
        store.close()


def test_teardown(app, flask_storm):
    ctx = app.app_context()
    ctx.push()

    real_store = flask_storm.store

    with patch.object(real_store, "close", wraps=real_store.close) as mock:
        # Assert teardown is not called before application context is gone
        assert not mock.called

        # Teardown application context
        ctx.pop()

        # Make sure store was closed after application context teardown
        assert mock.called

def test_teardown_binds(app, flask_storm):
    app.config["STORM_BINDS"] = {
        "extra": app.config["STORM_DATABASE_URI"]
    }

    ctx = app.app_context()
    ctx.push()

    real_store = flask_storm.get_store()
    extra_store = flask_storm.get_store("extra")

    patch_real_store = patch.object(real_store, "close", wraps=real_store.close)
    patch_extra_store = patch.object(
        extra_store, "close", wraps=extra_store.close)
    with patch_real_store as store_mock, patch_extra_store as extra_mock:
        # Assert teardown is not called before application context is gone
        assert not store_mock.called
        assert not extra_mock.called

        # Teardown application context
        ctx.pop()

        # Make sure stores were closed after application context teardown
        assert store_mock.called
        assert extra_mock.called


@require("flask_storm")
def test_shell_context(app):
    ctx = app.make_shell_context()
    try:
        assert "store" in ctx
        assert "_store_tracer" in ctx
    finally:
        ctx["_store_tracer"].stop()



import sys

from contextlib import contextmanager
from datetime import datetime
from flask import _app_ctx_stack, has_request_context, request
from operator import itemgetter
from storm.tracer import install_tracer, remove_tracer
from werkzeug.local import Local

from .sql import Adapter, replace_placeholders, format as format_sql, color as color_sql
from .utils import has_color_support, colored

try:
    import sqlparse
except ImportError:
    sqlparse = None


__all__ = [
    "DebugTracer",
    "get_debug_queries",
    "RequestTracer",
    "ShellTracer",
]


class DebugQuery(tuple):
    __slots__ = ()

    statement = property(itemgetter(0))
    params = property(itemgetter(1))
    start_time = property(itemgetter(2))
    end_time = property(itemgetter(3))

    @property
    def duration(self):
        if self.start_time is not None and self.end_time is not None:
            return self.end_time - self.start_time

    def __new__(cls, statement, params, start_time, end_time):
        return tuple.__new__(cls, [statement, params, start_time, end_time])


class DebugTracer(object):
    """
    A tracer which stores all queries with parameters onto the current request
    context. Queries are accessible using :func:`get_debug_queries`. It is used
    as a context manager. Since all queries are stored on the request context,
    they can only be accessed until the current request tears down.

    ::

        with DebugTracer():
            # Perform queries
            queries = get_debug_queries()

    .. note::
       :func:`get_debug_queries` do not need to be called within the context
       manager, as long as the request context is still alive, since all queries
       are stored on the request context.
    """

    def __init__(self):
        # Use thread locals since the tracer gets installed globally. This
        # ensures start time will be correctly measured, even in multi-threaded
        # environments. Werkzeug's implementation is used instead of threading
        # from the standard library since Werkzeug supports greenlets as well.
        self.threadinfo = Local()

    def connection_raw_execute(self, connection, raw_cursor, statement, params):
        self.threadinfo.start_time = datetime.now()

    def connection_raw_execute_success(
            self, connection, raw_cursor, statement, params):

        ctx = _app_ctx_stack.top
        if ctx is None:
            return

        if not hasattr(ctx, "storm_debug_queries"):
            ctx.storm_debug_queries = []

        ctx.storm_debug_queries.append(DebugQuery(
            statement,
            params,
            getattr(self.threadinfo, "start_time"),
            datetime.now()))

        # Remove start time to prevent leakage across queries
        delattr(self.threadinfo, "start_time")

    def connection_raw_execute_error(
            self, connection, raw_cursor, statement, params, error):
        self.connection_raw_execute_success(
            connection, raw_cursor, statement, params)

    def __enter__(self):
        install_tracer(self)

    def __exit__(self, type, exception, traceback):
        remove_tracer(self)


def get_debug_queries():
    """
    Return an array of queries executed within the context of a
    :class:`DebugTracer` under the current application context and thread.
    """

    return getattr(_app_ctx_stack.top, "storm_debug_queries", [])


class ShellTracer(object):
    """
    :param file: File like object (has write method) where queries will be
                 logged to.
    :param fancy: When ``True`` (default) colored output is used, if support is
                  detected, when ``False`` plain text is used.
    """

    def __init__(self, file=None, fancy=None):
        if file is None:
            self.file = sys.stdout
        else:
            self.file = file

        self.fancy = False
        if fancy is None:
            self.fancy = True

        # Use thread locals since the tracer gets installed globally. This
        # ensures start time will be correctly measured, even in multi-threaded
        # environments. Werkzeug's implementation is used instead of threading
        # from the standard library since Werkzeug supports greenlets as well.
        self.threadinfo = Local()
        self.threadinfo.active = False

    @property
    def use_color(self):
        return self.fancy and has_color_support(self.file)

    def _log(self, msg):
        if not self.threadinfo.active:
            return

        self.file.write(
            u"{};\n".format(color_sql(msg) if self.use_color else msg))

    def _log_result(self, success):
        if not self.threadinfo.active:
            return

        time = self.threadinfo.end_time - self.threadinfo.start_time
        msg = u"-- {result} in {time} ms".format(
            result="SUCCESS" if success else "FAILURE",
            time=time.total_seconds() * 1000)

        self.file.write(u"{}\n".format(
            colored(msg, 244) if self.use_color else msg))

    def connection_raw_execute(self, connection, raw_cursor, statement, params):
        # Start timer after log printing since it may delay query execution and
        # skew the time
        self.threadinfo.start_time = datetime.now()

        if sqlparse is None:
            self._log(u"{}\n{!r}".format(statement, params))
        else:
            self._log(format_sql(
                replace_placeholders(statement, params, Adapter(connection))))

    def connection_raw_execute_success(
            self, connection, raw_cursor, statement, params):
        self.threadinfo.end_time = datetime.now()
        self._log_result(True)

    def connection_raw_execute_error(
            self, connection, raw_cursor, statement, params, error):
        self.threadinfo.end_time = datetime.now()
        self._log_result(False)

    def start(self):
        """
        Install and activate this tracer for all statements executed in this
        thread (or greenlet).
        """

        # We also need to indicate which particular thread this is installed for
        # since it will write for all threads unless told not to
        self.threadinfo.active = True

        install_tracer(self)

    def stop(self):
        """
        Stop using this tracer.
        """

        remove_tracer(self)
        self.threadinfo.active = False

    def __enter__(self):
        self.start()

    def __exit__(self, type, exception, traceback):
        self.stop()


class RequestTracer(ShellTracer):
    """
    A tracer which prints all SQL queries generated by Storm directly to STDOUT.
    This is useful when debugging views in flask.

    ::

        with RequestTracer():
            # Queries executed here will be printed into STDOUT
            ...

    :param file: File like object (has write method) where queries will be
                 logged to.
    :param fancy: When ``True`` (default) colored output is used, if support is
                  detected, when ``False`` plain text is used.
    """

    def _get_request_line(self, request):
        template = (
            '{ip} - - [{timestamp:%d/%b/%Y %H:%M:%S}] '
            '"{method} {path} {version}" SQL -\n')

        full_path = request.path
        if request.args:
            full_path += "?" + request.environ.get("QUERY_STRING")

        return template.format(
            ip=request.remote_addr,
            timestamp=datetime.now(),
            method=request.method,
            path=full_path,
            version=request.environ.get("SERVER_PROTOCOL"))

    def _log(self, msg):
        if has_request_context():
            self.file.write(self._get_request_line(request))
        super(RequestTracer, self)._log(msg)



import pytest

def remove_whitespace():
    assert pytest.helpers.remove_whitespace("foo  bar") == "foobar"
    assert pytest.helpers.remove_whitespace("foo\t\tbar") == "foobar"
    assert pytest.helpers.remove_whitespace("foo\n\nbar") == "foobar"
    assert pytest.helpers.remove_whitespace("foo\r\rbar") == "foobar"
    assert pytest.helpers.remove_whitespace("foo\r\nbar") == "foobar"


def remove_ansi():
    assert pytest.helpers.remove_ansi("foo\x1b[0mbar") == "foobar"
    assert pytest.helpers.remove_ansi("foo\x1b[0bar") == "foo\x1b[0bar"



import datetime as dt
import os
import sys

from collections import MutableMapping
from flask import current_app
from functools import partial
from storm.variables import Variable
from werkzeug.local import LocalProxy


def find_flask_storm(app):
    """
    Find and return :py:class:`FlaskStorm` instance for the given application.

    :param app: Application to look for FlaskStorm instance in.
    :return: FlaskStorm instance if found, else None.
    """

    return getattr(app, "extensions", {}).get("storm")


def _lookup_storm_store(bind=None):
    app = current_app
    if not app:
        raise RuntimeError("Working outside an application context")

    flask_storm = find_flask_storm(app)
    if flask_storm is None:
        raise RuntimeError("FlaskStorm is not bound to the current application")

    return flask_storm.get_store(bind)


def create_context_local(bind):
    """
    Create a context local for the given bind. None means the default store.

    ::

        # Context local for the bind report_store
        report_store = create_context_local("report_store")

    This is what is used to implement the store context local which analogue to

    ::

        store = create_context_local(None)

    :param bind: Bind name of database URI to use when creating store.
    :raises RuntimeError: if working outside application context or if
                          FlaskStorm is not bound to the current application.

    """

    return LocalProxy(partial(_lookup_storm_store, bind))


def has_color_support(file=None):
    """
    Determine if terminal has color support. Code borrowed from
    django.core.management.color.supports_color().

    :return: True if color is supported, else False
    """

    if file is None:
        file = sys.stdout

    supported_platform = sys.platform != "Pocket PC" and (
        sys.platform != "win32" or "ANSICON" in os.environ)
    is_tty = getattr(file, "isatty", lambda: False)()

    return supported_platform and is_tty


def colored(msg, color=None, background=None, bold=None, underline=None):
    """
    Return string formatted using ANSI escape codes. Supports 256 colors.

    https://upload.wikimedia.org/wikipedia/en/1/15/Xterm_256color_chart.svg

    :param string: String to format.
    :param color: Color of text.
    :param background: Color of background.
    :param bold: True if text should be bold.
    :param underline: True if text should be underlined.
    """

    prefix = ""

    if color is not None:
        prefix += "\x1b[38;5;{}m".format(color)

    if background is not None:
        prefix += "\x1b[48;5;{}m".format(background)

    if bold:
        prefix += "\x1b[1m"

    if underline:
        prefix += "\x1b[4m"

    suffix = ""
    if prefix:
        suffix = "\x1b[0m"

    return "{}{}{}".format(prefix, msg, suffix)



import pytest
import re

from flask import Flask
from flask_storm import FlaskStorm


pytest_plugins = [
    "helpers_namespace",
]


@pytest.fixture
def app(request):
    app = Flask("foo")
    app.config["STORM_DATABASE_URI"] = "sqlite://:memory:"
    return app


@pytest.fixture
def flask_storm(app):
    # The app here will be the same app instance if requested by the test
    # function due to the scoping nature of py.test
    flask_storm = FlaskStorm()
    flask_storm.init_app(app)
    return flask_storm

@pytest.yield_fixture
def app_context(app):
    with app.app_context() as ctx:
        yield ctx


@pytest.helpers.register
def remove_whitespace(string):
    return re.sub(r"\s+", "", string)


@pytest.helpers.register
def remove_ansi(string):
    return re.sub(r"\x1b\[[^m]+m", "", string)



import pytest
import re
import sys

from datetime import datetime, timedelta
from flask_storm import store, FlaskStorm
from flask_storm.debug import DebugTracer, get_debug_queries, DebugQuery, ShellTracer
from mock import MagicMock, patch
from storm.exceptions import OperationalError
from threading import Thread

try:
    from io import StringIO
except ImportError:
    from StringIO import StringIO

# Enable skipping of tests that do not run without sqlparse being installed
try:
    import sqlparse
except ImportError:
    sqlparse = None

require_sqlparse = pytest.mark.skipif(not sqlparse, reason="requires sqlparse")

require = pytest.mark.usefixtures


@require("app", "flask_storm", "app_context")
def test_get_debug_queries():
    with DebugTracer():
        store.execute("SELECT 1")
        store.execute("SELECT ? + ?", [1, 2])

    queries = get_debug_queries()
    assert len(queries) == 2

    first, second = queries
    assert first.statement == "SELECT 1"
    assert first.params == ()
    assert isinstance(first.start_time, datetime)
    assert isinstance(first.end_time, datetime)
    assert isinstance(first.duration, timedelta)

    assert second.statement == "SELECT ? + ?"
    assert second.params == [1, 2]


def test_debug_query():
    data = (
        "SELECT 1",
        (),
        datetime(2000, 1, 1),
        datetime(2000, 1, 1, second=1))
    dq = DebugQuery(*data)

    for i, value in enumerate(data):
        assert dq[i] == value


def test_debug_query_keys():
    dq = DebugQuery(
        "SELECT 1",
        (),
        datetime(2000, 1, 1),
        datetime(2000, 1, 1, second=1))

    assert dq.statement == "SELECT 1"
    assert dq.params == ()
    assert dq.start_time == datetime(2000, 1, 1)
    assert dq.end_time == datetime(2000, 1, 1, second=1)
    assert dq.duration == timedelta(seconds=1)


@require("flask_storm")
def test_tracer_thread_isolation(app, app_context):
    with DebugTracer():
        store.execute("SELECT 'this'")

        # Spawn a separate thread that also executes a query
        def other_request():
            with app.app_context():
                store.execute("SELECT 'other'")

                # Ensure main thread does not leak into this one
                queries = get_debug_queries()
                assert len(queries) == 1
                assert queries[0].statement == "SELECT 'other'"

        t = Thread(target=other_request)
        t.start()
        t.join()

    # Ensure query log was not polluted by other thread
    queries = get_debug_queries()
    assert len(queries) == 1
    assert queries[0].statement == "SELECT 'this'"


def test_get_debug_queries_no_context():
    assert get_debug_queries() == []


def test_query_without_context(app):
    fs = FlaskStorm(app)
    try:
        store = fs.connect()
        with DebugTracer():
            store.execute("SELECT 1")

        # Since no application context is available queries are not saved
        # anywhere
        assert get_debug_queries() == []
    finally:
        store.close()


@require("app_context", "flask_storm")
def test_query_error():
    try:
        with DebugTracer():
            store.execute("SELECT !")
    except Exception:
        pass

    queries = get_debug_queries()
    assert len(queries) == 1
    assert queries[0].statement == "SELECT !"


@require_sqlparse
@require("app_context", "flask_storm")
def test_shell_tracer():
    # Alias helper functions
    remove_whitespace = pytest.helpers.remove_whitespace
    remove_ansi = pytest.helpers.remove_ansi

    sql = "SELECT 1 + 1 FROM (SELECT 2 + 2)"

    output = StringIO()
    output.isatty = MagicMock(return_value=True)

    with patch("sys.platform", "linux2"), ShellTracer(file=output, fancy=True):
        store.execute(sql)

    color_output = output.getvalue()
    output.close()

    assert color_output != sql
    assert \
        remove_whitespace(remove_ansi(color_output)).rsplit(";", 1)[0] == \
        remove_whitespace(sql)

    output = StringIO()
    output.isatty = MagicMock(return_value=False)

    with ShellTracer(file=output, fancy=False):
        store.execute(sql)

    colorless_output = output.getvalue()
    output.close()

    assert color_output != colorless_output
    assert \
        remove_ansi(color_output).rsplit(";", 1)[0] == \
        colorless_output.rsplit(";", 1)[0]
    assert \
        remove_whitespace(remove_ansi(color_output)).rsplit(";", 1)[0] == \
        remove_whitespace(sql)

def test_shell_tracer_defaults():
    st = ShellTracer()

    assert st.file is sys.stdout
    assert st.fancy


@require("app_context", "flask_storm")
def test_shell_tracer_success():
    output = StringIO()

    with ShellTracer(file=output, fancy=False):
        store.execute("SELECT 1")

    assert "SUCCESS" in output.getvalue()
    output.close()


@require("app_context", "flask_storm")
def test_shell_tracer_failure():
    output = StringIO()

    with ShellTracer(file=output, fancy=False), pytest.raises(Exception):
        store.execute("SELECT !")

    assert "FAILURE" in output.getvalue()
    output.close()



import pytest
import sys

from datetime import date
from flask_storm.sql import default_adapter, replace_placeholders, format, color
from mock import patch

# Enable skipping of tests that do not run without sqlparse being installed
try:
    import sqlparse
except ImportError:
    sqlparse = None

require_sqlparse = pytest.mark.skipif(not sqlparse, reason="requires sqlparse")


def test_adapter_type():
    assert default_adapter.type is None


def test_adapter_str():
    assert default_adapter.adapt(u"foobar") == u"'foobar'"
    assert default_adapter.adapt(u"foo'bar") == u"'foo''bar'"
    assert default_adapter.adapt(ur"foo\'bar") == ur"'foo\\''bar'"


def test_adapter_bool():
    assert default_adapter.adapt(True) == u"TRUE"
    assert default_adapter.adapt(False) == u"FALSE"


def test_adapter_none():
    assert default_adapter.adapt(None) == u"NULL"


def test_adapter_int():
    assert default_adapter.adapt(42) == u"42"
    assert default_adapter.adapt(sys.maxint) == str(sys.maxint)

    # Trigger long type
    assert default_adapter.adapt(sys.maxint + 1) == str(sys.maxint + 1)


def test_adapter_float():
    assert default_adapter.adapt(1.5) == u"1.5"


def test_adapter_date():
    d = date.today()
    assert default_adapter.adapt(d) == default_adapter.adapt(str(d))

@require_sqlparse
def test_replace_placeholders():
    assert replace_placeholders("SELECT ? + ?", [1, 2]) == "SELECT 1 + 2"
    assert replace_placeholders("SELECT ?  +  ?", [1, 2]) == "SELECT 1  +  2"


@require_sqlparse
def test_replace_placeholders_exausted_params():
    with pytest.raises(ValueError):
        replace_placeholders("SELECT ?", [])

@require_sqlparse
def test_format():
    sql = "SELECT * FROM test t WHERE t.id > 10"
    fsql = format(sql)

    assert fsql != sql
    assert \
        pytest.helpers.remove_whitespace(fsql) == \
        pytest.helpers.remove_whitespace(sql)


def test_format_no_sqlparse():
    sql = "SELECT * FROM test t WHERE t.id > 10"
    with patch("flask_storm.sql.sqlparse", None):
        assert sql == format(sql)

@require_sqlparse
def test_color():
    sql = """
        SELECT *
        FROM test t
        WHERE t.id > 10 AND t.name = 'foobar'
        ORDER BY lower(t.name)
    """
    csql = color(sql)

    assert csql != sql
    assert pytest.helpers.remove_ansi(csql) == sql


def test_color_no_sqlparse():
    sql = """
        SELECT *
        FROM test t
        WHERE t.id > 10 AND t.name = 'foobar'
        ORDER BY lower(t.name)
    """

    with patch("flask_storm.sql.sqlparse", None):
        assert sql == color(sql)



from storm.databases.postgres import PostgresConnection

try:
    import sqlparse
except ImportError:
    sqlparse = None

try:
    from psycopg2.extensions import adapt as psycopg2_adapt
except ImportError:
    psycopg2_adapt = None

from .utils import colored

__all__ = [
    "Adapter",
    "default_adapter",
    "replace_placeholders",
    "format",
    "color",
]

class Adapter(object):
    def __init__(self, conn=None):
        self._conn = conn

    @property
    def type(self):
        if isinstance(self._conn, PostgresConnection):
            return "postgres"

    def _default_adapt(self, value):
        if isinstance(value, (str, unicode)):
            return "'{}'".format(value.replace("'", "''").replace("\\", "\\\\"))
        elif isinstance(value, bool):
            return "TRUE" if value else "FALSE"
        elif value is None:
            return "NULL"
        elif isinstance(value, (int, long, float)):
            return str(value)
        else:
            return self._default_adapt(str(value))

    def adapt(self, value):
        if self.type == "postgres":
            output = psycopg2_adapt(value)
            output.prepare(conn._raw_connection)
            return output.getquoted()

        return self._default_adapt(value)


default_adapter = Adapter()


def replace_placeholders(statement, params, adapter=None):
    if adapter is None:
        adapter = default_adapter

    param_iter = (adapter.adapt(param) for param in params)
    tokens = []
    try:
        for token in sqlparse.parse(statement)[0].flatten():
            if str(token.ttype) == "Token.Name.Placeholder":
                tokens.append(param_iter.next())
            else:
                tokens.append(token.value)
    except StopIteration:
        raise ValueError("Not enough parameters provided")

    return "".join(tokens)


def format(statement):
    # If sqlparse is not installed it is not possible to do fancy formatting
    if sqlparse is None:
        return statement

    # To finish of we pretty print the output
    return sqlparse.format(
        statement,
        keyword_case="upper",
        identifier_case="lower",
        reindent=True,
        indent_tabs=False,
        indent_width=4)


def _color_token(token):
    Token = sqlparse.tokens
    if token.is_keyword or token.ttype is Token.Wildcard:
        return colored(token.value, 160, bold=True)
    elif token.ttype is Token.Punctuation:
        return colored(token.value, 246)
    elif isinstance(token, sqlparse.sql.Identifier):
        return colored(token.value, 69)
    elif token.ttype in Token.Literal.String:
        return colored(token.value, 220)
    elif token.ttype in Token.Literal.Number:
        return colored(token.value, 39)
    elif token.ttype in Token.Operator:
        return colored(token.value, 154)
    elif token.is_group:
        return "".join(_color_token(t) for t in token.tokens)
    return token.value


def color(statement):
    if sqlparse is None:
        return statement
    return "".join(_color_token(t) for t in sqlparse.parse(statement)[0].tokens)



import pytest

from flask_storm import store, create_context_local
from storm.locals import Store


require = pytest.mark.usefixtures


@require("app_context")
def test_context_local(flask_storm):
    assert isinstance(store._get_current_object(), Store)
    assert store._get_current_object() is flask_storm.store


def test_no_context():
    with pytest.raises(RuntimeError):
        store._get_current_object()


@require("app_context")
def test_wrong_context():
    with pytest.raises(RuntimeError):
        store._get_current_object()


def test_bind_context_local(app, flask_storm):
    app.config["STORM_BINDS"] = {
        "extra": app.config["STORM_DATABASE_URI"]
    }

    extra_store = create_context_local("extra")

    with app.app_context():
        assert extra_store._get_current_object() is flask_storm.get_store("extra")



from logging import getLogger, NullHandler

from .debug import DebugTracer, get_debug_queries, RequestTracer
from .ext import FlaskStorm
from .utils import find_flask_storm, create_context_local

logger = getLogger(__name__)
logger.addHandler(NullHandler())

__version__ = "0.1.1"

__all__ = [
    "create_context_local",
    "DebugTracer",
    "FlaskStorm",
    "find_flask_storm",
    "get_debug_queries",
    "RequestTracer",
    "store",
]

#: Shorthand for :attr:`FlaskStorm.store` which does not depend on knowing
#: the FlaskStorm instance bound to the current request context. This is the
#: prefered method of accessing the Store, since using the
#: :attr:`FlaskStorm.store` property directly makes it easy to accidentally
#: create circular imports.
store = create_context_local(None)



# example.py
from flask import Flask, jsonify
from flask_storm import FlaskStorm, store
from random import choice
from storm.locals import Int, Unicode


app = Flask("example")
app.config["STORM_DATABASE_URI"] = "sqlite:///test.db"

flask_storm = FlaskStorm()
flask_storm.init_app(app)


class Post(object):
    __storm_table__ = "posts"

    id = Int(primary=True)
    name = Unicode()
    text = Unicode()

    def __init__(self, name=None, text=None):
        if name is not None:
            self.name = name

        if text is not None:
            self.text = text


@app.cli.command()
def initdb():
    """Create schema and fill database with 15 sample posts by random authors"""

    store.execute("""DROP TABLE IF EXISTS posts""")
    store.execute("""
        CREATE TABLE posts(
            id    INTEGER PRIMARY KEY,
            name  VARCHAR,
            text  VARCHAR
        )
    """)

    names = [
        u"Alice",
        u"Bob",
        u"Eve",
    ]

    for i in range(1, 16):
        store.add(Post(choice(names), u"Post #{}".format(i)))

    store.commit()


@app.route("/")
def index():
    """Return the 10 latest posts in JSON format"""

    return jsonify([{
        "id": post.id,
        "name": post.name,
        "text": post.text,
    } for post in store.find(Post).order_by(Desc(Post.id)).config(limit=10)])



#!/usr/bin/env python
import re
import os

from setuptools import setup


class RstPreProcessor(object):
    def __init__(self):
        self.roles = {}
        self.blocks = {}
        self.replaces = {}

    def add_block(self, block, callback=None):
        if callback is None:
            def wrapper(callback):
                self.add_block(block, callback)
                return callback
            return wrapper
        self.blocks[block] = callback

    def add_role(self, role, callback=None):
        if callback is None:
            def wrapper(callback):
                self.add_role(role, callback)
                return callback
            return wrapper
        self.roles[role] = callback

    def add_replacement(self, search, replacement):
        self.replaces[search] = replacement

    def _block_dispatch(self, match):
        if match.group("block") not in self.blocks:
            return match.group(0)

        return self.blocks[match.group("block")](
            self,
            match.group("block"),
            match.group("args"),
            match.group("extra"),
            match.group("content"))

    def _role_dispatch(self, match):
        if match.group(1) not in self.roles:
            return match.group(0)

        return self.roles[match.group("role")](
            self,
            match.group("role"),
            match.group("args"),
            match.group("content"))

    def process(self, text):
        # Process blocks
        text = re.sub(
            "\.\.\s+(?:(?P<extra>\S+)\s+)?(?P<block>[^\n:]+)::"
            "\s+(?P<args>[^\n]+)(?:\n\n?(?P<content>.*?)\n\n(?=\S))?",
            self._block_dispatch,
            text,
            flags=re.DOTALL)

        # Process roles
        text = re.sub(
            ":(?P<role>[A-Za-z0-9_]+)(?:\s+(?P<args>[A-Za-z0-9_]+))?:"
            "`(?P<content>[^`]*)`",
            self._role_dispatch,
            text,
            flags=re.MULTILINE)

        # Run replaces
        for search, replacement in self.replaces.items():
            text = text.replace(search, replacement)

        return text


rst_pre_processor = RstPreProcessor()

@rst_pre_processor.add_role("class")
@rst_pre_processor.add_role("attr")
@rst_pre_processor.add_role("meth")
def role_simplyfier(processor, role, argument, content):
    format = {
        "attr": "``.{}``",
        "meth": "``{}()``",
    }

    if content.startswith("~"):
        return format.get(role, "``{}``").format(content[1:].split(".")[-1])
    else:
        return format.get(role, "``{}``").format(content + extra.get(role, ""))


@rst_pre_processor.add_block("include")
def includer(processor, block, args, extra, content):
    with open(args) as fp:
        return fp.read().rstrip()


with open("README.rst") as fp:
    long_desc = rst_pre_processor.process(fp.read())

with open("flask_storm/__init__.py") as fp:
    version = re.search('__version__\s+=\s+"([^"]+)', fp.read()).group(1)

if __name__ == "__main__":
    setup(
        name="Flask-Storm",
        version=version,
        description="Storm integration for Flask.",
        long_description=long_desc,
        license="MIT",
        author="Andreas Runfalk",
        author_email="andreas@runfalk.se",
        url="https://www.github.com/runfalk/flask_storm",
        packages=["flask_storm"],
        platforms="any",
        install_requires=[
            "Flask",
            "storm"
        ],
        extras_require={
            "dev": [
                "mock",
                "pytest>=3",
                "pytest-cov",
                "pytest-helpers-namespace",
                "sphinx",
            ],
            "fancy": [
                "sqlparse>=0.2.2",
            ],
        },
        classifiers=(
            "Development Status :: 3 - Alpha",
            "Framework :: Flask",
            "Intended Audience :: Developers",
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python",
            "Programming Language :: Python :: 2",
            "Topic :: Database",
            "Topic :: Utilities",
            "Topic :: Software Development :: Libraries",
        ),
        zip_safe=False,
    )



import pytest
import sys

from flask_storm.utils import colored, has_color_support
from mock import Mock, patch


fg42 = "\x1b[38;5;42m"
bg69 = "\x1b[48;5;69m"
bold = "\x1b[1m"
underline = "\x1b[4m"
reset = "\x1b[0m"


def j(*args):
    return "".join(args)


def test_reset():
    assert len(colored("")) == 0
    assert colored("", bold=True).endswith("\x1b[0m")


def test_foreground_color():
    assert colored("foo", color=42) == j(fg42, "foo", reset)


def test_background_color():
    assert colored("foo", background=69) == j(bg69, "foo", reset)


def test_bold():
    assert colored("foo", bold=True) == j(bold, "foo", reset)


def test_underline():
    assert colored("foo", underline=True) == j(underline, "foo", reset)


def test_prefix_order():
    ansi = colored("foo", color=42, background=69, bold=True, underline=True)
    real = j(fg42, bg69, bold, underline, "foo", reset)
    assert ansi == real


def test_has_color_support():
    tty_file = Mock()
    tty_file.isatty.return_value = True

    file = Mock()
    file.isatty.return_value = False

    with patch("sys.platform", "Pocket PC"):
        assert not has_color_support(tty_file)

        # This will check against STDOUT, but will always fail because of the
        # Pocket PC platform
        assert not has_color_support()

    with patch("sys.platform", "darwin"):
        assert has_color_support(tty_file)
        assert not has_color_support(file)

    with patch("sys.platform", "linux2"):
        assert has_color_support(tty_file)
        assert not has_color_support(file)

    with patch("sys.platform", "win32"):
        assert not has_color_support(tty_file)
        with patch.dict("os.environ", {"ANSICON": ""}):
            assert has_color_support(tty_file)




from flask import current_app, _app_ctx_stack
from storm.locals import create_database, Store

from .debug import ShellTracer
from .utils import find_flask_storm, create_context_local

class FlaskStorm(object):
    """
    Create a FlaskStorm instance.

    :param app: Application to enable Flask-Storm for. This is the same as
                calling :py:meth:`~FlaskStorm.init_app` after initialization.
    """

    _app = None

    def __init__(self, app=None):
        if app is not None:
            self.init_app(app)

        # Order is important here as init_app() checks that self._app is None
        self._app = app

    @property
    def app(self):
        if self._app is not None:
            return self._app
        return current_app

    @property
    def is_bound(self):
        return self._app is not None

    def init_app(self, app):
        """
        Binds this extension to the given application. This is important since
        the database connection will not close unless the application has been
        initialized.

        .. note::
           One instance of FlaskStorm may be registered to multiple
           applications. One application should however only be registered to
           one FlaskStorm instance, or tear down functionality will trigger
           multiple times.

        :param app: Application to enable Flask-Storm for.
        :raises RuntimeError: if an application is already bound to this
                              instance by being passed to the constructor.
        """

        # If this class was instanciated with an app as its argument this method
        # we fail because this instance is not dependent on application context
        if self.is_bound:
            raise RuntimeError(
                "This FlaskStorm instance is already strictly bound to an "
                "application. To use this instance with multiple applications "
                "do not pass an application to the constructor and call "
                "init_app() for every application")

        # Check if there is an existing FlaskStorm instance bound to this app
        flask_storm = find_flask_storm(app)
        if flask_storm is not None:
            raise ValueError(
                "FlaskStorm already registered for this application")

        # Register instance of self on the application object. This dictionary
        # has existed on the Flask class since 0.7, and we rely on the
        # application context, which came later. Hence there is no check for
        # whether the attribute exists or not
        app.extensions["storm"] = self

        # Register a shell context function to make the store application
        # context locally available. This also installs a debug tracer which
        # prints every query to STDOUT. This function is available from version
        # 0.11 of Flask and later
        if hasattr(app, "shell_context_processor"):
            @app.shell_context_processor
            def shell_context():
                tracer = ShellTracer(fancy=True)
                tracer.start()

                return {
                    "store": create_context_local(None),
                    "_store_tracer": tracer,
                }

        # Register teardown to make sure Store is closed at the end of each
        # request
        @app.teardown_appcontext
        def close_store(response_or_exception):
            ctx = _app_ctx_stack.top
            for store in getattr(ctx, "storm_store", {}).values():
                store.close()

    def get_binds(self):
        """
        Return dict of database URIs for the application as defined by the
        ``STORM_BINDS`` configuration variable. If ``STORM_DATABASE_URI`` is
        defined it will be available using the key ``None``.

        :return: Dict from bind names to database URIs.
        """

        binds = self.app.config.get("STORM_BINDS", {}).copy()
        # Check if None key exists since that is used for the default store
        if None in binds:
            raise RuntimeError(
                "There is a None key in STORM_BINDS. This is reserved for "
                "the default store as defined by STORM_DATABASE_URI")

        if "STORM_DATABASE_URI" in self.app.config:
            binds[None] = self.app.config.get("STORM_DATABASE_URI")

        return binds

    def connect(self, bind=None):
        """
        Return a new Store instance with a connection to the database specified
        in the STORM_DATABASE_URI configuration variable of the bound application.
        This method normally not be called externally as it does not close the
        store once the application context tears down.

        :param bind: Database URI to use, defaults to ``STORM_DATABASE_URI``.
                     See :py:meth:`.get_binds` for details.
        :return: Store instance
        :raises RuntimeError: if no connection URI is found.
        """

        binds = self.get_binds()
        if bind not in binds:
            raise RuntimeError(
                "No connection URI found in configuration. Is "
                "STORM_DATABASE_URI defined?")

        return Store(create_database(binds[bind]))

    def get_store(self, bind=None):
        """
        Return a Store instance for the current application context. If there is
        no instance a new one will be created. Instances created using this
        method will close on application context tear down.

        :param bind: Bind name of database URI. Defaults to the one specified by
                     ``STORM_DATABASE_URI``.
        :return: Store for the current application context.
        :raises RuntimeError: if accessed outside the scope of an application
                              context.
        """

        ctx = _app_ctx_stack.top
        if ctx is not None:
            # Create a dictionary where open store connections are stored
            if not hasattr(ctx, "storm_store"):
                ctx.storm_store = {}

            storm_store = ctx.storm_store
            if bind not in ctx.storm_store:
                ctx.storm_store[bind] = self.connect(bind)
            return ctx.storm_store[bind]

    store = property(get_store)
